from __future__ import annotations

MODELS: list[str] = [
    "deepseek-v3",
    "deepseek-v3.1",
    "deepseek-v3.2",
    "kimi-k2-thinking",
    "qvq-max",
    "qvq-plus",
    "qwen-1.8b-chat",
    "qwen-1.8b-longcontext-chat",
    "qwen-14b-chat",
    "qwen-72b-chat",
    "qwen-7b-chat",
    "qwen-coder-plus",
    "qwen-coder-turbo",
    "qwen-deep-search-planning",
    "qwen-flash",
    "qwen-flash-character",
    "qwen-long",
    "qwen-math-plus",
    "qwen-math-turbo",
    "qwen-max",
    "qwen-max-longcontext",
    "qwen-mt-flash",
    "qwen-mt-lite",
    "qwen-mt-plus",
    "qwen-mt-turbo",
    "qwen-omni-turbo",
    "qwen-plus",
    "qwen-turbo",
    "qwen-vl-max",
    "qwen-vl-ocr",
    "qwen-vl-plus",
    "qwen1.5-0.5b-chat",
    "qwen1.5-1.8b-chat",
    "qwen1.5-110b-chat",
    "qwen1.5-14b-chat",
    "qwen1.5-32b-chat",
    "qwen1.5-72b-chat",
    "qwen1.5-7b-chat",
    "qwen2-0.5b-instruct",
    "qwen2-1.5b-instruct",
    "qwen2-57b-a14b-instruct",
    "qwen2-7b-instruct",
    "qwen2.5-0.5b-instruct",
    "qwen2.5-1.5b-instruct",
    "qwen2.5-14b-instruct",
    "qwen2.5-14b-instruct-1m",
    "qwen2.5-32b-instruct",
    "qwen2.5-3b-instruct",
    "qwen2.5-72b-instruct",
    "qwen2.5-7b-instruct",
    "qwen2.5-7b-instruct-1m",
    "qwen2.5-coder-0.5b-instruct",
    "qwen2.5-coder-14b-instruct",
    "qwen2.5-coder-32b-instruct",
    "qwen2.5-coder-3b-instruct",
    "qwen2.5-coder-7b-instruct",
    "qwen2.5-math-1.5b-instruct",
    "qwen2.5-math-72b-instruct",
    "qwen2.5-math-7b-instruct",
    "qwen2.5-vl-32b-instruct",
    "qwen3-0.6b",
    "qwen3-1.7b",
    "qwen3-14b",
    "qwen3-235b-a22b",
    "qwen3-235b-a22b-instruct-2507",
    "qwen3-235b-a22b-thinking-2507",
    "qwen3-30b-a3b",
    "qwen3-30b-a3b-instruct-2507",
    "qwen3-30b-a3b-thinking-2507",
    "qwen3-32b",
    "qwen3-4b",
    "qwen3-8b",
    "qwen3-coder-480b-a35b-instruct",
    "qwen3-coder-flash",
    "qwen3-coder-plus",
    "qwen3-livetranslate-flash",
    "qwen3-livetranslate-flash-realtime",
    "qwen3-max-preview",
    "qwen3-next-80b-a3b-instruct",
    "qwen3-next-80b-a3b-thinking",
    "qwen3-omni-flash",
    "qwen3-omni-flash-realtime",
    "qwen3-vl-flash",
    "qwen3-vl-plus",
    "qwq-plus",
    "qwen3-asr-flash-realtime",
    "qwen-image-edit-plus",
    "qwen-image-max",
    "z-image-turbo",
    "qwen-tts",
    "qwen3-tts-flash",
    "qwen3-tts-flash-realtime",
    "qwen3-tts-vc-realtime",
    "qwen3-tts-vd-realtime",
]
