from __future__ import annotations

MODELS: list[str] = [
    "claude-haiku-4-5-20251001",
    "claude-haiku-4-5-20251001-thinking",
    "claude-sonnet-4-5-20250929",
    "claude-sonnet-4-5-20250929-thinking",
    "claude-opus-4-5-20251101",
    "claude-opus-4-5-20251101-thinking",
    "deepseek-chat",
    "deepseek-coder",
    "deepseek-r1",
    "deepseek-r1-all",
    "deepseek-r1-search",
    "deepseek-r1-search-pro",
    "deepseek-r1-search-pro-thinking",
    "deepseek-search",
    "deepseek-v3",
    "deepseek-v3.2",
    "deepseek-v3.2-exp-thinking",
    "deepseek-v3.2-thinking",
    "doubao-seed-1-8-251228",
    "doubao-seed-1-8-251228-thinking",
    "gemini-2.5-flash",
    "gemini-2.5-flash-deepsearch",
    "gemini-2.5-pro",
    "gemini-2.5-pro-deepsearch",
    "gemini-3-flash-preview",
    "gemini-3-flash-preview-all",
    "gemini-3-flash-preview-thinking",
    "gemini-3-pro",
    "gemini-3-pro-deepsearch",
    "gemini-3-pro-preview",
    "gemini-3-pro-preview-thinking",
    "gemini-3-pro-thinking",
    "glm-4.5",
    "glm-4.5-air",
    "glm-4.5-airx",
    "glm-4.5-flash",
    "glm-4.5-x",
    "glm-4v",
    "gpt-4o",
    "gpt-4o-mini",
    "gpt-4o-search",
    "gpt-4o-study",
    "gpt-5",
    "gpt-5-mini",
    "gpt-5-nano",
    "gpt-5-pro",
    "gpt-5-thinking-all",
    "gpt-5.1",
    "gpt-5.1-thinking",
    "gpt-5.1-thinking-all",
    "gpt-5.2",
    "gpt-oss-120b",
    "gpt-oss-20b",
    "grok-3",
    "grok-3-deepersearch",
    "grok-3-deepsearch",
    "grok-3-reasoner",
    "grok-3-think",
    "grok-4",
    "grok-4.1",
    "kimi-k2-instruct",
    "o1",
    "o1-mini",
    "o1-pro",
    "o1-pro-all",
    "o3",
    "o3-all",
    "o3-mini",
    "o3-mini-high",
    "o3-pro",
    "o4-mini",
    "qwen2.5-72b-instruct",
    "qwen3-235b-a22b",
    "qwen3-32b",
    "search",
    "search-gpts",
    "search-gpts-chat",
    "dall-e-3",
    "doubao-seedream-4-0-250828",
    "doubao-seedream-4-5-251128",
    "flux-kontext-max",
    "flux-kontext-pro",
    "gemini-2.5-flash-image",
    "gemini-3-pro-image-preview",
    "gemini-3-pro-image-preview-2k",
    "gemini-3-pro-image-preview-4k",
    "gpt-image-1",
    "gpt-image-1.5",
    "kling_image",
    "seededit",
    "seedream-3.0",
    "seedream-4-0-250828",
    "seedream-v4",
    "kling-video-o1",
    "kling-video-o1-edit",
    "kling_video",
    "sora-16:9-480p-10s",
    "sora-16:9-480p-5s",
    "sora-16:9-720p-5s",
    "sora-1:1-480p-10s",
    "sora-1:1-480p-5s",
    "sora-1:1-720p-5s",
    "sora-2",
    "sora-2-character",
    "sora-2-pro",
    "sora-2-pro-character",
    "sora-9:16-480p-10",
    "sora-9:16-480p-5s",
    "sora-9:16-720p-5s",
    "veo2",
    "veo2-fast",
    "veo2-fast-frames",
    "veo2-pro",
    "veo3",
    "veo3-fast",
    "veo3-fast-frames",
    "veo3-frames",
    "veo3-pro",
    "veo3-pro-frames",
    "veo3.1",
    "veo3.1-components",
    "veo3.1-pro",
    "chirp-auk",
    "chirp-bluejay",
    "chirp-crow",
    "chirp-v3-0",
    "chirp-v3-5",
    "chirp-v4",
    "sonar-medium-chat",
    "sonar-medium-online",
    "sonar-small-chat",
    "sonar-small-online",
]
