from __future__ import annotations

MODELS: list[str] = [
    "aqa",
    "deep-research-pro-preview-12-2025",
    "gemini-2.0-flash",
    "gemini-2.0-flash-lite",
    "gemini-2.5-computer-use-preview-10-2025",
    "gemini-2.5-flash",
    "gemini-2.5-flash-lite",
    "gemini-2.5-flash-native-audio-latest",
    "gemini-2.5-pro",
    "gemini-3-flash-preview",
    "gemini-3-pro-preview",
    "gemini-exp-1206",
    "gemini-flash-latest",
    "gemini-flash-lite-latest",
    "gemini-pro-latest",
    "gemini-robotics-er-1.5-preview",
    "gemma-3-12b-it",
    "gemma-3-1b-it",
    "gemma-3-27b-it",
    "gemma-3-4b-it",
    "gemma-3n-e2b-it",
    "gemma-3n-e4b-it",
    "gemini-2.5-flash-image",
    "gemini-3-pro-image-preview",
    "imagen-4.0-fast-generate-001",
    "imagen-4.0-generate-001",
    "imagen-4.0-generate-preview-06-06",
    "imagen-4.0-ultra-generate-001",
    "veo-2.0-generate-001",
    "veo-3.0-fast-generate-001",
    "veo-3.0-generate-001",
    "veo-3.1-fast-generate-preview",
    "veo-3.1-generate-preview",
    "gemini-2.5-flash-preview-tts",
    "gemini-2.5-pro-preview-tts",
    "embedding-001",
    "embedding-gecko-001",
    "gemini-embedding-001",
    "gemini-embedding-exp",
    "gemini-embedding-exp-03-07",
    "text-embedding-004",
]
