from __future__ import annotations

MODELS: list[str] = [
    "gpt-4.1",
    "gpt-4.1-mini",
    "gpt-4.1-nano",
    "gpt-4o",
    "gpt-4o-mini",
    "chatgpt-4o-latest",
    "gpt-5",
    "gpt-5-codex",
    "gpt-5-mini",
    "gpt-5-nano",
    "gpt-5-pro",
    "gpt-5.1",
    "gpt-5.1-codex",
    "gpt-5.1-codex-max",
    "gpt-5.1-codex-mini",
    "gpt-5.2",
    "gpt-5.2-pro",
    "o1",
    "o1-pro",
    "o3",
    "o3-mini",
    "o4-mini",
    "gpt-4o-mini-transcribe",
    "gpt-4o-transcribe",
    "gpt-4o-transcribe-diarize",
    "whisper-1",
    "dall-e-2",
    "dall-e-3",
    "gpt-image-1",
    "gpt-image-1-mini",
    "gpt-image-1.5",
    "chatgpt-image-latest",
    "sora-2",
    "sora-2-pro",
    "gpt-4o-mini-tts",
    "tts-1",
    "tts-1-hd",
    "text-embedding-3-large",
    "text-embedding-3-small",
    "text-embedding-ada-002",
]
