from __future__ import annotations

MODELS: list[str] = [
    "claude-instant-1.2",
    "claude-3-5-haiku-latest",
    "claude-3-5-sonnet-latest",
    "claude-3-7-sonnet-latest",
    "claude-3-7-sonnet-latest-thinking",
    "claude-sonnet-4-0",
    "claude-sonnet-4-0-thinking",
    "claude-opus-4-0",
    "claude-opus-4-0-thinking",
    "claude-opus-4-1",
    "claude-haiku-4-5",
    "claude-sonnet-4-5",
    "claude-opus-4-5",
    "Cursor-c4",
    "Cursor-c4-thinking",
    "Cursor-co4",
    "Cursor-co4-thinking",
]
