from __future__ import annotations

MODELS: list[str] = [
    "deepseek-r1-250120",
    "deepseek-r1-250528",
    "deepseek-r1-distill-qwen-32b-250120",
    "deepseek-r1-distill-qwen-7b-250120",
    "deepseek-v3-1-250821",
    "deepseek-v3-1-terminus",
    "deepseek-v3-2-251201",
    "deepseek-v3-241226",
    "deepseek-v3-250324",
    "doubao-1-5-lite-32k-250115",
    "doubao-1-5-pro-256k-250115",
    "doubao-1-5-pro-32k-250115",
    "doubao-1-5-pro-32k-character-250228",
    "doubao-1-5-pro-32k-character-250715",
    "doubao-1-5-thinking-pro-250415",
    "doubao-1-5-thinking-pro-m-250415",
    "doubao-1-5-thinking-pro-m-250428",
    "doubao-1-5-thinking-vision-pro-250428",
    "doubao-1-5-ui-tars-250428",
    "doubao-1-5-vision-pro-32k-250115",
    "doubao-1.5-ui-tars-250328",
    "doubao-1.5-vision-lite-250315",
    "doubao-1.5-vision-pro-250328",
    "doubao-embedding-vision-241215",
    "doubao-embedding-vision-250328",
    "doubao-embedding-vision-250615",
    "doubao-embedding-vision-251215",
    "doubao-lite-128k-240428",
    "doubao-lite-128k-240828",
    "doubao-lite-32k-240428",
    "doubao-lite-32k-240628",
    "doubao-lite-32k-240828",
    "doubao-lite-32k-character-241015",
    "doubao-lite-32k-character-250228",
    "doubao-lite-4k-240328",
    "doubao-lite-4k-character-240515",
    "doubao-lite-4k-character-240828",
    "doubao-lite-4k-pretrain-character-240516",
    "doubao-pro-128k-240515",
    "doubao-pro-128k-240628",
    "doubao-pro-256k-241115",
    "doubao-pro-32k-240615",
    "doubao-pro-32k-240828",
    "doubao-pro-32k-241215",
    "doubao-pro-32k-browsing-240615",
    "doubao-pro-32k-browsing-240828",
    "doubao-pro-32k-browsing-241115",
    "doubao-pro-32k-character-240528",
    "doubao-pro-32k-character-240828",
    "doubao-pro-32k-character-241215",
    "doubao-pro-32k-functioncall-240515",
    "doubao-pro-32k-functioncall-240815",
    "doubao-pro-32k-functioncall-241028",
    "doubao-pro-32k-functioncall-preview",
    "doubao-pro-4k-240515",
    "doubao-pro-4k-browsing-240524",
    "doubao-pro-4k-character-240515",
    "doubao-pro-4k-character-240728",
    "doubao-pro-4k-functioncall-240515",
    "doubao-pro-4k-functioncall-240615",
    "doubao-seaweed-241128",
    "doubao-seed-1-6-250615",
    "doubao-seed-1-6-251015",
    "doubao-seed-1-6-flash-250615",
    "doubao-seed-1-6-flash-250715",
    "doubao-seed-1-6-flash-250828",
    "doubao-seed-1-6-lite-251015",
    "doubao-seed-1-6-thinking-250615",
    "doubao-seed-1-6-thinking-250715",
    "doubao-seed-1-6-vision-250815",
    "doubao-seed-1-8-251228",
    "doubao-seed-code-preview-251028",
    "doubao-seed-translation-250915",
    "doubao-seed3d-1-0-250928",
    "doubao-seedance-1-0-pro-250528",
    "doubao-seedance-1-0-pro-fast-251015",
    "doubao-seedance-1-5-pro-251215",
    "doubao-seededit-3-0-i2i-250628",
    "doubao-smart-router-250928",
    "doubao-vision-lite-32k-241015",
    "doubao-vision-pro-32k-241028",
    "glm-4-5-air-20250728",
    "kimi-k2-250711",
    "kimi-k2-250905",
    "kimi-k2-thinking-251104",
    "mistral-7b-instruct-v0.2",
    "qwen2-5-72b-20240919",
    "qwen3-0-6b-20250429",
    "qwen3-14b-20250429",
    "qwen3-32b-20250429",
    "qwen3-8b-20250429",
    "wan2-1-14b-flf2v-250417",
    "wan2-1-14b-i2v-250225",
    "wan2-1-14b-t2v-250225",
    "doubao-seedream-3-0-t2i-250415",
    "doubao-seedream-4-0-250828",
    "doubao-seedream-4-5-251128",
    "doubao-seedance-1-0-lite-i2v-250428",
    "doubao-seedance-1-0-lite-t2v-250428",
    "doubao-embedding-large-text-240915",
    "doubao-embedding-large-text-250515",
    "doubao-embedding-text-240515",
    "doubao-embedding-text-240715",
]
