from __future__ import annotations

MODELS: list[str] = [
    "gemini-2.5-flash",
    "gemini-2.5-flash-lite",
    "gemini-2.5-flash-nothink",
    "gemini-2.5-flash-search",
    "gemini-2.5-pro",
    "gemini-2.5-pro-search",
    "gemini-3-flash-preview",
    "gemini-3-pro-preview",
    "gemini-3-pro-preview-search",
    "gemini-2.0-flash-exp-image-generation",
    "gemini-2.0-flash-preview-image-generation",
    "gemini-2.5-flash-image",
    "gemini-3-pro-image-preview",
    "gemini-embedding-001",
    "gemini-embedding-exp-03-07",
]
